#ifndef KYEXECTL_WLIST_H
#define KYEXECTL_WLIST_H

#include <stddef.h>
#include <stdint.h>
#include <sys/queue.h>

/**
 * @brief 白名单数据库记录
 *
 */
struct exectl_obj {
    char *fpath;            /**< 文件路径 */
    char *hash;             /**< 摘要值 */
    int ftype;              /**< 文件类型 */
    int dtype;              /**< 数据类型 */
    int status;             /**< 数据状态 */
};

/**
 * @brief 白名单数据项构体
 */
struct exectl_obj_entry {
    struct exectl_obj obj;
    SLIST_ENTRY(exectl_obj_entry) entries;
};
/**
 * @brief 执行控制单链表结构体
 *
 * 单链表结构体名字是exectl_obj_list，链表成员类型是exectl_obj_entry
 */
SLIST_HEAD(exectl_obj_list, exectl_obj_entry);

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief 创建白名单链表结构，需要使用 exectl_objs_free 释放 list 内存
 *
 * @param list 白名单链表
 * @param obj 白名单数据项
 *
 * @return  成功 返回 0，失败 返回 -1
 */
extern int exectl_list_entry_create(struct exectl_obj_list *list, struct exectl_obj *obj);

/**
 * @brief 释放白名单链表内存
 *
 * @param objs 白名单链表
 */
extern void exectl_objs_free(struct exectl_obj_list *objs);

/**
 * @brief 释放白名单数据项内存
 *
 * @param obj 白名单数据项
 */
extern void exectl_obj_free(struct exectl_obj *obj);

/**
 * @brief 添加文件到白名单
 *
 * @param path 要添加到白名单的文件的路径
 *
 * @return 0 成功 其他 失败
 */
extern int exectl_wlist_add(const char *path);

/**
 * @brief 批量添加文件到白名单
 *
 * @param paths 要添加到白名单的文件的路径数组
 * @param count 文件数量
 *
 * @return  >= 0 添加成功的数量，其他 失败
 */
extern int exectl_wlist_multi_add(char *paths[], size_t count);

/**
 * @brief 遍历目录添加文件到白名单
 *
 * @param dir 要遍历的目录的路径
 *
 * @return  >= 0 添加成功的数量，其他 失败
 */
extern int exectl_wlist_add_by_dir(const char *dir);

/**
 * @brief 更新白名单中数据
 *
 * @param path 要更新的文件的路径
 *
 * @return  0 成功，其他 失败
 */
extern int exectl_wlist_update(const char *path);

/**
 * @brief 批量更新白名单中数据
 *
 * @param paths 要更新的文件的路径数组
 * @param count 文件数量
 *
 * @return  >= 0 更新成功的数量，其他 失败
 */
extern int exectl_wlist_multi_update(char *paths[], size_t count);

/**
 * @brief 遍历目录更新白名单
 *
 * @param dir 要遍历的目录的路径
 *
 * @return  >= 0 更新成功的数量，其他 失败
 */
extern int exectl_wlist_update_by_dir(const char *dir);

/**
 * @brief 从白名单中删除数据
 *
 * @param path 要删除的文件的路径
 *
 * @return  0 成功，其他 失败
 */
extern int exectl_wlist_delete(const char *path);

/**
 * @brief 批量删除白名单中数据
 *
 * @param paths 要删除的文件的路径数组
 * @param count 路径个数
 *
 * @return  >= 0 删除成功的数量，其他 失败
 */
extern int exectl_wlist_multi_delete(char *paths[], size_t count);

/**
 * @brief 遍历目录删除白名单
 *
 * @param dir 要遍历的目录的路径
 *
 * @return  >= 0 删除成功的数量，其他 失败
 */
extern int exectl_wlist_delete_by_dir(const char *dir);

/**
 * @brief 获取白名单数据
 *
 * @param key 数据关键词
 * @param objs 白名单链表，存储数据，使用完后需要使用 exectl_objs_free 释放内存
 *
 * @return  0 成功，其他 失败
 */
extern int exectl_wlist_load(const char *key, struct exectl_obj_list *objs);

/**
 * @brief 获取对应路径的白名单数据
 *
 * @param path 要获取数据的文件的路径
 * @param obj 白名单数据项，存储数据，使用完后需要使用 exectl_obj_free 释放内存
 *
 * @return  0 成功，其他 失败
 */
extern int exectl_wlist_load_by_path(const char *path, struct exectl_obj *obj);

/**
 * @brief 获取所有白名单数据
 *
 * @param objs 白名单链表，存储数据，使用完后需要使用 exectl_objs_free 释放内存
 *
 * @return  0 成功，其他 失败
 */
extern int exectl_wlist_load_all(struct exectl_obj_list *objs);

/**
 * @brief 获取白名单中对应文件的基准值
 *
 * @param path 要获取基准值的文件的路径
 *
 * @return  成功 返回文件的基准值，否则 返回 NULL
 */
extern char *exectl_wlist_get_hash_by_path(const char *path);

/**
 * @brief 修改数据库对于文件的数据项的 status 字段
 *
 * @param path 要修改的文件的路径
 * @param status 执行控制保护状态：0 正常，1 已篡改
 *
 * @return  0 成功，其他 失败
 */
extern int exectl_wlist_change_status(const char *path, int status);

/**
 * @brief 批量修改数据库对于文件的数据项的 status 字段
 *
 * @param paths 要修改的文件的路径数组
 * @param count 路径个数
 * @param status 执行控制保护状态：0 正常，1 已篡改
 *
 * @return  0 成功，其他 失败
 */
extern int exectl_wlist_multi_change_status(char *paths[], size_t count, int status);

#ifdef __cplusplus
}
#endif

#endif /* KYEXECTL_WLIST_H */

